#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# 2025/02/05
# Author: Adwin <adwin@host.com.tw>
# Description: 快速設定網頁保護 cPanel 使用者網站，且避免因還原或使用者手動刪除而失效
import os
import sys
import json
import subprocess

SUSPEND_BODY = """
<Directory "$DOC_ROOT">
    AllowOverride FileInfo Indexes Limit Options
    AllowOverrideList Require
    AuthType Basic
    AuthName "Abuse - TWNOC $ABUSE_REASON"
    AuthUserFile "/etc/apache2/conf.d/includes/website_abuse_passwd"
    require valid-user
</Directory>
"""

ABUSE_PASSWD = "/etc/apache2/conf.d/includes/website_abuse_passwd"
ABUSE_CONF = "/etc/apache2/conf.d/includes/website_abuse.conf"


def check_is_blocked(docroot: str) -> bool:
    if docroot == "":
        return False

    with open(ABUSE_CONF, "r") as f:
        if docroot in f.read():
            return True

    return False


def get_cpanel_user_home(username: str) -> str:
    a = run_cmd(
        f"uapi --output=jsonpretty --user={username} Variables get_user_information"
    )
    try:
        return json.loads(a)["result"]["data"]["home"]
    except json.decoder.JSONDecodeError:
        return ""
    except Exception as e:
        print(f"Error: {e}")
        return ""


def run_cmd(command: str, display: bool = False) -> str:
    if display:
        print(command)

    result = subprocess.run(
        command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )
    return result.stdout.decode("utf-8", errors="ignore").strip()


def fast_suspend_website() -> None:
    if len(sys.argv) < 2:
        print("Usage: fast_suspend_website.py cPanel帳號 封鎖原因")
        return

    if len(sys.argv) >= 2:
        username = sys.argv[1]
        reason = "No reason"

    if len(sys.argv) >= 3:
        reason = sys.argv[2]

    # 取得 cPanel 使用者的根目錄
    docroot = get_cpanel_user_home(username)

    # 檢查使用者是否已經被封鎖
    if check_is_blocked(docroot):
        print(f"{username} 已有設定網頁保護")
        return

    # 如果返回空字串，表示使用者不存在
    if docroot == "":
        print(f"{username} 此 cPanel 帳號不存在")
        return

    # 確認是否有 /etc/apache2/conf.d/includes/website_abuse_passwd
    # 如果沒有，則建立一個帳號 user 密碼為 xKSx4JYLE8
    if not os.path.exists(ABUSE_PASSWD):
        with open(ABUSE_PASSWD, "w") as f:
            f.write("user:$apr1$cSQmQAFL$fHIWoZ69uFTm/U652jHuQ/")

    # 確認是否有 /etc/apache2/conf.d/includes/website_abuse.conf
    if os.path.exists(ABUSE_CONF) and os.path.isfile(ABUSE_CONF):
        with open(ABUSE_CONF, "a") as f:
            f.write(
                SUSPEND_BODY.replace("$DOC_ROOT", docroot).replace(
                    "$ABUSE_REASON", reason
                )
            )
    else:
        with open(ABUSE_CONF, "w") as f:
            f.write(
                SUSPEND_BODY.replace("$DOC_ROOT", docroot).replace(
                    "$ABUSE_REASON", reason
                )
            )

    # 重啟 apache
    run_cmd("/usr/local/cpanel/scripts/restartsrv_httpd")

    print(f"{username} 已設定網頁保護並已重啟 Apache")


if __name__ == "__main__":
    fast_suspend_website()
